/*!
 * @file orxonox.h
 * Orxonox core functions
*/

#ifndef _ORXONOX_H
#define _ORXONOX_H

#include "base_object.h"

class WorldEntity;
class GameLoader;
class IniParser;

//! orxonox core singleton class
/**
*/
class Orxonox : public BaseObject {

 public:
  virtual ~Orxonox ();
  /** @returns a Pointer to the only object of this Class */
  inline static Orxonox* getInstance() { if (!singletonRef) singletonRef = new Orxonox();  return singletonRef; };

  int init (int argc, char** argv);

  void restart();

  void start();

  //void graphicsHandler (SDL_Event* event);

 private:
  Orxonox ();

  void parseIniFile(const char* fileName);

  int initResources ();
  int initVideo ();
  int initSound ();
  int initInput ();
  int initNetworking ();
  int initMisc ();


  const char* getConfigFile ();

 private:
  static Orxonox*   singletonRef;            //!< singleton reference to orxonox

  IniParser*        iniParser;               //!< Reference to the ini-parser used in orxonox
  char*             configFileName;          //!< Filename of the configuration-file.
  GameLoader*       gameLoader;              //!< The gameLoader

  unsigned int      argc;                    //!< Count of Arguments of orxonox
  char**            argv;                    //!< Values of th Arguments of orxonox.
};


////////////////////////
// Start-up functions //
////////////////////////
int startHelp(int argc, char** argv);
int startOrxonox(int argc, char** argv);

#endif /* _ORXONOX_H */

